@@ -3,8 +3,9 @@ from xl import main as ex, trax, common, event, xdg, settings, providers
 from xl import player
 from xl.trax import search, util
 from xl.nls import gettext as _
-from xlgui import oldmenu as menu
-from xlgui import panel, playlist
+from xlgui import panel
+from xlgui.panel import menus
+from xlgui.widgets.notebook import NotebookPage
 from xlgui.widgets.common import DragTreeView
 import HTMLParser
 from StringIO import StringIO
@@ -17,7 +18,7 @@ import glib
 import gobject
 import gtk
 import os
-import pylast
+from xl.externals import pylast
 import contextprefs
 import re
 import urllib
@@ -34,31 +35,6 @@ PANEL = None
 CURPATH = os.path.realpath(__file__)
 BASEDIR = os.path.dirname(CURPATH)+os.path.sep
 
-class ContextPopup(menu.TrackSelectMenu):
-
-    def __init__(self, widget):
-        menu.TrackSelectMenu.__init__(self)
-        self.widget = widget
-
-    def on_append_items(self, selected=None):
-        """
-            Appends the selected tracks to the current playlist
-        """
-        selected = self.widget.get_selected_tracks()
-        pl = xlgui.get_controller().main.get_selected_page()
-        if pl:
-            pl.playlist.add_tracks(selected)
-
-    def on_queue(self, selected=None):
-        """
-            Called when the user clicks the "toggle queue" item
-        """
-        selected = self.widget.get_selected_tracks()
-        pl = xlgui.get_controller().main.get_selected_page()
-        ex.exaile().queue.add_tracks(selected)
-        if pl:
-            pl.playlist.add_tracks(selected)
-            pl.list.queue_draw()
 
 class BrowserPage(webkit.WebView, providers.ProviderHandler):
 
@@ -69,6 +45,9 @@ class BrowserPage(webkit.WebView, providers.ProviderHandler):
         webkit.WebView.__init__(self)
         providers.ProviderHandler.__init__(self, "context_page")
 
+        # HACK: so that the panel menu works
+        self.tree = self
+        
         self.connect_events()
         self.hover = ''
         self.theme = theme
@@ -185,7 +164,13 @@ class BrowserPage(webkit.WebView, providers.ProviderHandler):
         self.refresh_button_image.set_from_stock(gtk.STOCK_REFRESH, 1)
 
     def on_field_refresh(self, type=None, obj=None, data=None):
-        self.execute_script(self.refresh_script % (data[0], u'%s' % data[1].replace('"', '\\"').replace('\n', '\\\n'), data[0]))
+        '''
+           This is frequently called on a different thread, should 
+           always execute the script on the main thread. If not, 
+           errors may occur
+        '''
+        script = self.refresh_script % (data[0], u'%s' % data[1].replace('"', '\\"').replace('\n', '\\\n'), data[0])
+        glib.idle_add(self.execute_script, script)
 
     def push(self, page):
         self.history = self.history[:self.history_index+1]
@@ -236,7 +221,7 @@ class BrowserPage(webkit.WebView, providers.ProviderHandler):
     def button_press(self, widget, event):
         if event.button==3:
             if self.hover:
-                ContextPopup(self).popup(event)
+                menus.TrackPanelMenu(self).popup(event)
                 return True
             else:
                 return True
@@ -263,6 +248,9 @@ class BrowserPage(webkit.WebView, providers.ProviderHandler):
             self.drag_source_set_icon_stock(gtk.STOCK_DND_MULTIPLE)
         elif len(selection)>0: self.drag_source_set_icon_stock(gtk.STOCK_DND)
         return False
+    
+    def get_selection_empty(self):
+        return self.hover.split('://')[0] in ['track', 'artist', 'album', 'tag']
 
     def get_selected_tracks(self):
         tmp = self.hover.split('://', 1)[1]
@@ -1140,14 +1128,14 @@ class LyricsPage(PlayingPage):
     def __init__(self, theme, track, base='lyrics://', template='lyrics.html', async=[]):
         PlayingPage.__init__(self, theme, track, base, template, async)
 
-class ContextPanel(gobject.GObject):
+class ContextPanel(panel.Panel):
     """
         The contextual panel
     """
     ui_info = (BASEDIR+'context.ui', 'ContextPanelWindow')
 
     def __init__(self, parent):
-        self.init__(parent, _('Context'))
+        panel.Panel.__init__(self, parent, 'contextinfo')
 
         cachedir = os.path.join(xdg.get_data_dirs()[0], 'context')
         if not os.path.isdir(cachedir):
@@ -1159,25 +1147,12 @@ class ContextPanel(gobject.GObject):
 
         self.controller = parent
 
-        self.theme = ContextTheme(settings.get_option('context/theme', 'classic'))
+        self._theme = ContextTheme(settings.get_option('context/theme', 'classic'))
 
-        self._browser = BrowserPage(self.builder, self.theme)
+        self._browser = BrowserPage(self.builder, self._theme)
 
         self.setup_widgets()
 
-    def init__(self, parent, name=None):
-        """
-            Intializes the panel
-
-            @param controller: the main gui controller
-        """
-        gobject.GObject.__init__(self)
-        self.name = name
-        self.parent = parent
-        self.builder = gtk.Builder()
-        self.builder.add_from_file(self.ui_info[0])
-        self._child = None
-
     def setup_widgets(self):
         self._scrolled_window = gtk.ScrolledWindow()
         self._scrolled_window.props.hscrollbar_policy = gtk.POLICY_AUTOMATIC
@@ -1187,17 +1162,6 @@ class ContextPanel(gobject.GObject):
 
         frame = self.builder.get_object('ContextFrame')
         frame.add(self._scrolled_window)
-
-    def get_panel(self):
-        if not self._child:
-            window = self.builder.get_object(self.ui_info[1])
-            self._child = window.get_child()
-            window.remove(self._child)
-            if not self.name:
-                self.name = window.get_title()
-            window.destroy()
-
-        return (self._child, self.name)
         
 def get_preferences_pane():
     return contextprefs
@@ -1205,7 +1169,7 @@ def get_preferences_pane():
 def exaile_ready(object=None, a=None, b=None):
     global PANEL
     PANEL = ContextPanel(xlgui.get_controller().main)
-    xlgui.get_controller().add_panel(*PANEL.get_panel())
+    providers.register('main-panel', PANEL)
     
     event.remove_callback(exaile_ready, 'exaile_loaded')
 
@@ -1224,4 +1188,4 @@ def disable(exaile):
         Removes tab
     """
     global PANEL
-    xlgui.get_controller().remove_panel(PANEL.get_panel()[0])
+    providers.unregister('main-panel', PANEL)
