@@ -68,12 +68,19 @@ def migrate_settings():
             
         settings.set_option( migrated_option, True )
   
+def get_tagname():
+    return settings.get_option(tagname_option, 'grouping') 
+  
   
 def get_track_groups(track):
     '''
         Returns a set() of groups present in this track
-    '''
-    grouping = track.get_tag_raw(settings.get_option(tagname_option, 'grouping'), True)
+    '''  
+    return _get_track_groups(track, get_tagname())
+    
+    
+def _get_track_groups(track, tagname):
+    grouping = track.get_tag_raw(tagname, True)
     
     if grouping is not None:
         return set([ group.replace('_', ' ') for group in grouping.split()])
@@ -89,7 +96,7 @@ def set_track_groups(track, groups):
     '''
     
     grouping = ' '.join( sorted( [ '_'.join( group.split() ) for group in groups ] ) )
-    track.set_tag_raw(settings.get_option(tagname_option, 'grouping'), grouping )
+    track.set_tag_raw(get_tagname(), grouping )
     
     if not track.write_tags():
         dialogs.error( None, "Error writing tags to %s" % gobject.markup_escape_text(track.get_loc_for_io()) )
@@ -148,7 +155,7 @@ def _create_search_playlist( name, search_string, exaile ):
 def create_all_search_playlist( groups, exaile ):
     '''Create a playlist of tracks that have all groups selected'''
     
-    tagname = settings.get_option(tagname_option, 'grouping')
+    tagname = get_tagname()
     
     name = '%s: %s' % (tagname.title(), ' and '.join(groups))
     search_string = ' '.join( [ '%s~"\\b%s\\b"' % (tagname, re.escape(group.replace(' ','_'))) for group in groups ] )
