@@ -77,14 +77,16 @@ class CollectionScanThread(common.ProgressThread):
     """
         Scans the collection
     """
-    def __init__(self, collection):
+    def __init__(self, collection, startup_scan=False):
         """
             Initializes the thread
 
             :param collection: the collection to scan
+            :param startup_scan: Only scan libraries scanned at startup
         """
         common.ProgressThread.__init__(self)
-
+        
+        self.startup_scan = startup_scan
         self.collection = collection
 
     def stop(self):
@@ -101,7 +103,7 @@ class CollectionScanThread(common.ProgressThread):
         event.add_callback(self.on_scan_progress_update,
             'scan_progress_update')
 
-        self.collection.rescan_libraries()
+        self.collection.rescan_libraries(startup_only=self.startup_scan)
 
         event.remove_callback(self.on_scan_progress_update,
             'scan_progress_update')
@@ -236,7 +238,7 @@ class Collection(trax.TrackDB):
         """
         return self.libraries.values()
 
-    def rescan_libraries(self):
+    def rescan_libraries(self, startup_only=False):
         """
             Rescans all libraries associated with this Collection
         """
@@ -256,6 +258,10 @@ class Collection(trax.TrackDB):
         scan_interval = 20
 
         for library in self.libraries.itervalues():
+            
+            if startup_only and not (library.monitored and library.startup_scan):
+                continue
+            
             event.add_callback(self._progress_update, 'tracks_scanned',
                 library)
             library.rescan(notify_interval=scan_interval)
@@ -319,6 +325,7 @@ class Collection(trax.TrackDB):
             l['monitored'] = v.monitored
             l['realtime'] = v.monitored
             l['scan_interval'] = v.scan_interval
+            l['startup_scan'] = v.startup_scan
             _serial_libraries.append(l)
         return _serial_libraries
 
@@ -331,7 +338,7 @@ class Collection(trax.TrackDB):
         for l in _serial_libraries:
             self.add_library( Library( l['location'],
                     l.get('monitored', l.get('realtime')),
-                    l['scan_interval'] ))
+                    l['scan_interval'], l.get('startup_scan', True) ))
 
     _serial_libraries = property(serialize_libraries, unserialize_libraries)
 
@@ -385,7 +392,7 @@ class LibraryMonitor(gobject.GObject):
         self.__root = gio.File(library.location)
         self.__monitored = False
         self.__monitors = {}
-        self.__queue = []
+        self.__queue = {}
         self.__lock = threading.RLock()
 
     def do_get_property(self, property):
@@ -395,7 +402,7 @@ class LibraryMonitor(gobject.GObject):
         if property.name == 'monitored':
             return self.__monitored
         else:
-            raise AttributeError('unkown property %s' % property.name)
+            raise AttributeError('unknown property %s' % property.name)
 
     def do_set_property(self, property, value):
         """
@@ -433,26 +440,35 @@ class LibraryMonitor(gobject.GObject):
                     self.emit('location-removed', directory)
 
                 self.__monitors = {}
+                
+    def __process_change_queue(self, gfile):
+        if gfile in self.__queue:
+            added_tracks = trax.util.get_tracks_from_uri(gfile.get_uri())
+            self.__library.collection.add_tracks(added_tracks)
+            del self.__queue[gfile]
 
     def on_location_changed(self, monitor, gfile, other_gfile, event):
         """
             Updates the library on changes of the location
         """
         if event == gio.FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
-            if gfile in self.__queue:
-                added_tracks = trax.util.get_tracks_from_uri(gfile.get_uri())
-                self.__library.collection.add_tracks(added_tracks)
-                self.__queue.remove(gfile)
+            self.__process_change_queue(gfile)
         elif event == gio.FILE_MONITOR_EVENT_CREATED:
             # Enqueue tracks retrieval
             if gfile not in self.__queue:
-                self.__queue += [gfile]
-
+                self.__queue[gfile] = True
+            
+                # File monitor only emits the DONE_HINT when using inotify,
+                # and only on single files. Give it some time, but don't 
+                # lose the change notification
+                glib.timeout_add(500, self.__process_change_queue, gfile)
+            
             # Set up new monitor if directory
             fileinfo = gfile.query_info('standard::type')
 
             if fileinfo.get_file_type() == gio.FILE_TYPE_DIRECTORY and \
                gfile not in self.__monitors:
+                
                 for directory in common.walk_directories(gfile):
                     monitor = directory.monitor_directory()
                     monitor.connect('changed', self.on_location_changed)
@@ -507,7 +523,7 @@ class Library(object):
         5
         >>>
     """
-    def __init__(self, location, monitored=False, scan_interval=0):
+    def __init__(self, location, monitored=False, scan_interval=0, startup_scan=False):
         """
             Sets up the Library
 
@@ -523,6 +539,7 @@ class Library(object):
         self.scan_interval = scan_interval
         self.scan_id = 0
         self.scanning = False
+        self._startup_scan = startup_scan
         self.monitor = LibraryMonitor(self)
         self.monitor.props.monitored = monitored
 
@@ -595,6 +612,16 @@ class Library(object):
             self.scan_id = glib.timeout_add_seconds(interval, self.rescan)
 
         self.scan_interval = interval
+        
+    def get_startup_scan(self):
+        return self._startup_scan
+    
+    def set_startup_scan(self, value):
+        self._startup_scan = value
+        self.collection.serialize_libraries()
+        self.collection._dirty = True
+    
+    startup_scan = property(get_startup_scan, set_startup_scan)
 
     def _count_files(self):
         """
@@ -709,7 +736,7 @@ class Library(object):
         if self.scanning:
             return
 
-        logger.info("Scanning library: %s" % self.location)
+        logger.info("Scanning library: %s", self.location)
         self.scanning = True
         db = self.collection
         libloc = gio.File(self.location)
@@ -760,6 +787,7 @@ class Library(object):
 
             if self.collection and self.collection._scan_stopped:
                 self.scanning = False
+                logger.info("Scan canceled")
                 return
 
             # progress update
@@ -792,6 +820,8 @@ class Library(object):
         for tr in removals:
             logger.debug(u"Removing %s"%unicode(tr))
             self.collection.remove(tr)
+            
+        logger.info("Scan completed: %s", self.location)
         self.scanning = False
 
     def add(self, loc, move=False):
