@@ -186,7 +186,7 @@ class Exaile(object):
             from xl import settings
         except common.VersionError:
             common.log_exception(log=logger)
-            exit(1)
+            sys.exit(1)
             
         logger.debug("Settings loaded from %s" % settings.location)
         
@@ -236,12 +236,13 @@ class Exaile(object):
 
         # Initialize plugin manager
         from xl import plugins
+        self.plugins = plugins.PluginsManager(self)
+        
         if not self.options.SafeMode:
             logger.info("Loading plugins...")
-            self.plugins = plugins.PluginsManager(self)
+            self.plugins.load_enabled()
         else:
             logger.info("Safe mode enabled, not loading plugins.")
-            self.plugins = plugins.PluginsManager(self, load=False)
 
         # Initialize the collection
         logger.info("Loading collection...")
@@ -251,7 +252,7 @@ class Exaile(object):
                     location=os.path.join(xdg.get_data_dir(), 'music.db'))
         except common.VersionError:
             common.log_exception(log=logger)
-            exit(1)
+            sys.exit(1)
 
         from xl import event
         # Set up the player and playback queue
@@ -277,11 +278,26 @@ class Exaile(object):
         self.devices = devices.DeviceManager()
         event.log_event("device_manager_ready", self, None)
 
-        # Initialize HAL
+        # Initialize dynamic device discovery interface
+        # -> if initialized and connected, then the object is not None
+        
+        self.udisks2 = None
+        self.udisks = None
+        self.hal = None
+        
         if self.options.Hal:
             from xl import hal
-            self.hal = hal.HAL(self.devices)
-            self.hal.connect()
+                
+            udisks2 = hal.UDisks2(self.devices)
+            if udisks2.connect():
+                self.udisks2 = udisks2
+            else:
+                udisks = hal.UDisks(self.devices)
+                if udisks.connect():
+                    self.udisks = udisks
+                else:
+                    self.hal = hal.HAL(self.devices)
+                    self.hal.connect()
         else:
             self.hal = None
 
@@ -315,6 +331,10 @@ class Exaile(object):
                 self.gui.open_uri(args[0], play=True)
             for arg in args[1:]:
                 self.gui.open_uri(arg)
+            
+            # kick off autoscan of libraries
+            # -> don't do it in command line mode, since that isn't expected
+            self.gui.rescan_collection_with_progress(True)
 
         if restore:
             player.QUEUE._restore_player_state(
@@ -615,7 +635,7 @@ class Exaile(object):
     def version(self):
         from xl.version import __version__
         print "Exaile", __version__
-        exit()
+        sys.exit(0)
 
     def _add_default_playlists(self):
         """
@@ -672,6 +692,48 @@ class Exaile(object):
             Returns the current version
         """
         return __version__
+    
+    def get_user_agent_string(self, plugin_name=None):
+        '''
+            Returns an approrpiately formatted User-agent string for 
+            web requests. When possible, plugins should use this to
+            format user agent strings.
+            
+            Users can control this agent string by manually setting
+            general/user_agent and general/user_agent_w_plugin in settings.ini
+            
+            :param plugin_name: the name of the plugin
+        '''
+        
+        version = __version__
+        if '+' in version:  # strip out bzr tags
+            version = version[:version.index('+')]
+        
+        fmt = {
+            'version': version
+        }
+        
+        if not hasattr(self, '_user_agent_no_plugin'):
+                    
+            from xl import settings
+            
+            default_no_plugin = 'Exaile/%(version)s (+http://www.exaile.org)'
+            default_plugin = 'Exaile/%(version)s %(plugin_name)s/%(plugin_version)s (+http://www.exaile.org)'
+        
+            self._user_agent_no_plugin = \
+                settings.get_option('general/user_agent', default_no_plugin)
+            self._user_agent_w_plugin = \
+                settings.get_option('general/user_agent_w_plugin', default_plugin)
+        
+        if plugin_name is not None:
+            plugin_info = self.plugins.get_plugin_info(plugin_name)
+            
+            fmt['plugin_name'] = plugin_info['Name'].replace(' ', '')
+            fmt['plugin_version'] = plugin_info['Version']
+            
+            return self._user_agent_w_plugin % fmt
+        else:
+            return self._user_agent_no_plugin % fmt
 
     def quit(self, restart=False):
         """
@@ -734,7 +796,7 @@ class Exaile(object):
 
         logger.info("Bye!")
         logging.shutdown()
-        exit()
+        sys.exit(0)
 
 def exaile():
     if not Exaile._exaile:
