@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #
 #
 # The developers of the Exaile media player hereby grant permission
@@ -492,6 +492,8 @@ class AudioStream(gst.Bin):
             gst.SEEK_TYPE_NONE, 0)
 
         self.vol.send_event(seekevent)
+        # update this in case we're paused
+        self.last_position = value
 
     def _seek_delayed(self, type, object, value):
         """
