@@ -52,6 +52,7 @@ from xl import (
     common,
     dynamic,
     event,
+    main,
     providers,
     settings,
     trax,
@@ -59,6 +60,7 @@ from xl import (
 )
 from xl.common import MetadataList
 from xl.nls import gettext as _
+from xl.metadata.tags import tag_data
 
 logger = logging.getLogger(__name__)
 
@@ -68,6 +70,9 @@ class InvalidPlaylistTypeError(Exception):
 class PlaylistExists(Exception):
     pass
 
+class UnknownPlaylistTrackError(Exception):
+    pass
+
 PlaylistExportOptions = namedtuple('PlaylistExportOptions', 'relative')
 
 def encode_filename(filename):
@@ -366,12 +371,14 @@ class M3UConverter(FormatConverter):
         """
         playlist = Playlist(name=self.name_from_path(path))
         extinf = {}
+        lineno = 0
 
         logger.debug('Importing M3U playlist: %s' % path)
         
         with closing(gio.DataInputStream(gio.File(path).read())) as stream:
             while True:
                 line = stream.read_line()
+                lineno += 1
 
                 if not line:
                     break
@@ -408,7 +415,12 @@ class M3UConverter(FormatConverter):
                     if extinf:
                         for tag, value in extinf.iteritems():
                             if track.get_tag_raw(tag) is None:
-                                track.set_tag_raw(tag, value)
+                                try:
+                                    track.set_tag_raw(tag, value)
+                                except Exception, e:
+                                    # Python 3: raise UnknownPlaylistTrackError() from e
+                                    # Python 2: .. no good solution 
+                                    raise UnknownPlaylistTrackError("line %s: %s" % (lineno, e))
 
                     playlist.append(track)
                     extinf = {}
@@ -1794,9 +1806,15 @@ class SmartPlaylist(object):
         if not collection: #if there wasnt one set we might not have one
             return
 
-        search_string = self._create_search_string()
+        search_string, matchers = self._create_search_data(collection)
 
         matcher = trax.TracksMatcher(search_string, case_sensitive=False)
+        
+        # prepend for now, since it is likely to remove more tracks, and
+        # smart playlists don't support mixed and/or expressions yet
+        for m in matchers:
+            matcher.prepend_matcher(m, self.or_match)
+        
         trs = [ t.track for t in trax.search_tracks(collection, [matcher]) ]
         if self.random_sort:
             random.shuffle(trs)
@@ -1813,12 +1831,13 @@ class SmartPlaylist(object):
 
         return pl
 
-    def _create_search_string(self):
+    def _create_search_data(self, collection):
         """
-            Creates a search string based on the internal params
+            Creates a search string + matchers based on the internal params
         """
 
         params = [] # parameter list
+        matchers = [] # matchers list
         maximum = settings.get_option('rating/maximum', 5)
         durations = {
             _('seconds'): lambda value: timedelta(seconds=value),
@@ -1833,15 +1852,30 @@ class SmartPlaylist(object):
                 params += [param]
                 continue
             (field, op, value) = param
+            fieldtype = tag_data.get(field)
             s = ""
 
             if field == '__rating':
                 value = float((100.0*value)/maximum)
-            elif field in ('__date_added', '__last_played'):
+            elif fieldtype == 'timestamp':
                 duration, unit = value
                 delta = durations[unit](duration)
                 point = datetime.now() - delta
                 value = time.mktime(point.timetuple())
+            elif field == '__playlist':
+                try:
+                    pl = main.exaile().playlists.get_playlist(value)
+                except:
+                    try:
+                        pl = main.exaile().smart_playlists.get_playlist(value).get_playlist(collection) 
+                    except Exception as e:
+                        raise ValueError("Loading %s: %s" % (self.name, str(e)))
+                    
+                if op == 'pin':
+                    matchers.append(trax.TracksInList(pl))
+                else:
+                    matchers.append(trax.TracksNotInList(pl))
+                continue
 
             if op == ">=" or op == "<=":
                 s += '( %(field)s%(op)s%(value)s ' \
@@ -1887,9 +1921,9 @@ class SmartPlaylist(object):
             params.append(s)
 
         if self.or_match:
-            return ' | '.join(params)
+            return ' | '.join(params), matchers
         else:
-            return ' '.join(params)
+            return ' '.join(params), matchers
 
     def save_to_location(self, location):
         pdata = {}
@@ -2018,7 +2052,7 @@ class PlaylistManager(object):
                 encode_filename(name)))
             return pl
         else:
-            raise ValueError("No such playlist")
+            raise ValueError("No such playlist '%s'" % name)
 
     def list_playlists(self):
         """
