@@ -48,50 +48,16 @@ from xlgui import (
     guiutil,
     icons,
     panel,
-    playlist,
     xdg
 )
+
+from xlgui.panel import menus
 from xlgui.widgets.common import DragTreeView
-from xlgui.widgets import (
-    menu,
-    menuitems
-)
+
 
 logger = logging.getLogger(__name__)
 
 
-def __create_files_panel_context_menu():
-    items = []
-    items.append(menuitems.AppendMenuItem('append', after=[]))
-    items.append(menuitems.ReplaceCurrentMenuItem('replace', after=[items[-1].name]))
-    items.append(menuitems.EnqueueMenuItem('enqueue', after=[items[-1].name]))
-    items.append(menu.simple_separator('sep1', after=[items[-1].name]))
-    items.append(menuitems.OpenDirectoryMenuItem('open-drectory', after=[items[-1].name]))
-    def trash_tracks_func(parent, context, tracks):
-        menuitems.generic_trash_tracks_func(parent, context, tracks)
-        parent.refresh(None)
-    items.append(menuitems.TrashMenuItem('trash-tracks',
-        after=[items[-1].name], trash_tracks_func=trash_tracks_func))
-    items.append(menu.simple_separator('sep2', after=[items[-1].name]))
-    items.append(menuitems.PropertiesMenuItem('properties', after=[items[-1].name]))
-    for item in items:
-        providers.register('files-panel-context-menu', item)
-__create_files_panel_context_menu()
-
-class FilesContextMenu(menu.ProviderMenu):
-    def __init__(self, panel):
-        menu.ProviderMenu.__init__(self, 'files-panel-context-menu', panel)
-
-    def get_context(self):
-        context = common.LazyDict(self._parent)
-        def get_selected_tracks(name, parent):
-            return parent.tree.get_selected_tracks() or []
-        context['selected-tracks'] = get_selected_tracks
-        context['selection-count'] = lambda name, parent: \
-            parent.tree.get_selection_count()
-        
-        return context
-
 class FilesPanel(panel.Panel):
     """
         The Files panel
@@ -104,11 +70,11 @@ class FilesPanel(panel.Panel):
 
     ui_info = ('files.ui', 'FilesPanelWindow')
 
-    def __init__(self, parent, collection):
+    def __init__(self, parent, collection, name):
         """
             Initializes the files panel
         """
-        panel.Panel.__init__(self, parent)
+        panel.Panel.__init__(self, parent, name)
         self.collection = collection
 
         self.box = self.builder.get_object('files_box')
@@ -117,7 +83,7 @@ class FilesPanel(panel.Panel):
 
         self._setup_tree()
         self._setup_widgets()
-        self.menu = FilesContextMenu(self)
+        self.menu = menus.FilesContextMenu(self)
 
         self.key_id = None
         self.i = 0
@@ -432,13 +398,13 @@ class FilesPanel(panel.Panel):
             for sortname, name, f in subdirs:
                 model.append((f, self.directory, name, ''))
             for sortname, name, f in subfiles:
-                size = f.query_info('standard::size').get_size() // 1024
+                size = f.query_info('standard::size').get_size() // 1000
                 
                 # locale.format_string does not support unicode objects
                 # correctly, so we call it with an str and convert the 
                 # locale-dependent output to unicode.
                 size = locale.format_string('%d', size, True)
-                size = _('%s KB') % unicode(size, locale.getpreferredencoding())
+                size = _('%s kB') % unicode(size, locale.getpreferredencoding())
                 
                 model.append((f, self.track, name, size))
 
@@ -487,13 +453,10 @@ class FilesDragTreeView(DragTreeView):
     """
         Custom DragTreeView to retrieve data from files
     """
-    def get_selection_count(self):
-        '''
-            Returns the number of items currently selected in the
-            playlist. Prefer this to len(get_selected_tracks()) et al
-            if you will discard the actual track list
-        '''
-        return self.get_selection().count_selected_rows()
+    
+    def get_selection_empty(self):
+        '''Returns True if there are no selected items'''
+        return self.get_selection().count_selected_rows() == 0
     
     def get_selected_tracks(self):
         """
@@ -507,7 +470,7 @@ class FilesDragTreeView(DragTreeView):
             f = model[path][0]
             self.append_recursive(tracks, f)
 
-        return tracks or None
+        return tracks
 
     def append_recursive(self, songs, f):
         """
