@@ -24,16 +24,15 @@
 # do so. If you do not wish to do so, delete this exception statement
 # from your version.
 
-import os.path
-import time
-import urllib
 
 import glib
 import gobject
-import gtk
+import gtk 
 
 from xl import (
+    common,
     event,
+    main,
     playlist,
     radio,
     settings,
@@ -41,15 +40,15 @@ from xl import (
 )
 from xl.nls import gettext as _
 from xlgui import (
-    guiutil,
     icons,
-    oldmenu as menu,
-    panel,
-    playlist as guiplaylist,
-    xdg
+    panel
 )
+from xlgui.panel import menus
+from xlgui.widgets import (
+    dialogs
+)
+
 from xlgui.widgets.common import DragTreeView
-from xlgui.widgets import dialogs
 from xlgui.widgets.filter import *
 
 def N_(x): return x
@@ -66,6 +65,14 @@ class EntryAndEntryField(MultiEntryField):
 class EntryDaysField(MultiEntryField):
     def __init__(self):
         MultiEntryField.__init__(self, (50, _('days')))
+        
+class PlaylistField(ComboEntryField):
+    def __init__(self):
+        playlists = []
+        playlists.extend(main.exaile().smart_playlists.list_playlists())
+        playlists.extend(main.exaile().playlists.list_playlists())
+        playlists.sort()
+        ComboEntryField.__init__(self, playlists)
 
 DATE_FIELDS = [
     N_('seconds'), N_('minutes'), N_('hours'), N_('days'), N_('weeks')]
@@ -86,105 +93,83 @@ class SpinNothing(SpinLabelField):
     def __init__(self):
         SpinLabelField.__init__(self, '')
 
+# This sets up the CRITERIA for all the available types of tags
+# that exaile supports. The actual CRITERIA dict is populated 
+# using xl.metadata.tags.tag_data.
+#
 # NOTE: The following strings are already marked for translation in _TRANS and
 # _NMAP, and will be really translated by filtergui; no need to clutter the
 # code here.
+__criteria_types = {
+    
+    # TODO              
+    'bitrate': None,
+                    
+    'image': None,
+          
+    'int': [
+        ('is', SpinNothing),
+        ('less than', SpinNothing),
+        ('greater than', SpinNothing),
+        ('between', EntryAndEntryField),
+        ('at least', SpinNothing),
+        ('at most', SpinNothing),
+        ('is set', NullField),
+        ('is not set', NullField),
+    ],
+          
+    'location': [
+        ('is', QuotedEntryField),
+        ('is not', QuotedEntryField),
+        ('contains', QuotedEntryField),
+        ('does not contain', QuotedEntryField),
+        ('regex', QuotedEntryField),
+        ('not regex', QuotedEntryField),
+    ],
+    
+    'text': [
+        ('is', EntryField),
+        ('is not', EntryField),
+        ('contains', EntryField),
+        ('does not contain', EntryField),
+        ('regex', EntryField),
+        ('not regex', EntryField),
+        ('is set', NullField),
+        ('is not set', NullField),
+    ],
+    
+    'time': [
+        ('at least', SpinSecondsField),
+        ('at most', SpinSecondsField),
+        ('is', SpinSecondsField),
+        ('is not', SpinSecondsField),
+    ],
+                    
+    'timestamp': [
+        ('in the last', SpinDateField),
+        ('not in the last', SpinDateField),
+    ],  
+}
 
+# aliases
+__criteria_types['datetime'] = __criteria_types['text'] # TODO: fix 
+__criteria_types['multiline'] = __criteria_types['text']
+__criteria_types['dblnum'] = __criteria_types['int']
+
+
+# This gets populated below. Only add special tags/searches here.
 CRITERIA = [
-    ('Artist', [
-        ('is', EntryField),
-        ('is not', EntryField),
-        ('contains', EntryField),
-        ('does not contain', EntryField),
-        ('regex', EntryField),
-        ('not regex', EntryField),
-        ('is set', NullField),
-        ('is not set', NullField),
-    ]),
-    ('Album', [
-        ('is', EntryField),
-        ('is not', EntryField),
-        ('contains', EntryField),
-        ('does not contain', EntryField),
-        ('regex', EntryField),
-        ('not regex', EntryField),
-        ('is set', NullField),
-        ('is not set', NullField),
-    ]),
-    ('Title', [
-        ('is', EntryField),
-        ('is not', EntryField),
-        ('contains', EntryField),
-        ('does not contain', EntryField),
-        ('regex', EntryField),
-        ('not regex', EntryField),
-        ('is set', NullField),
-        ('is not set', NullField),
-    ]),
-    ('Genre', [
-        ('is', EntryField),
-        ('is not', EntryField),
-        ('contains', EntryField),
-        ('does not contain', EntryField),
-        ('is set', NullField),
-        ('is not set', NullField),
-    ]),
     ('Rating', [
         ('greater than', SpinRating),
         ('less than', SpinRating),
         ('at least', SpinRating),
         ('at most', SpinRating),
     ]),
-    ('Plays', [
-        ('at least', SpinNothing),
-        ('at most', SpinNothing),
-    ]),
-    ('Year', [
-        ('before', EntryField),
-        ('after', EntryField),
-        ('between', EntryAndEntryField),
-        ('is set', NullField),
-        ('is not set', NullField),
-    ]),
-    ('Length', [
-        ('at least', SpinSecondsField),
-        ('at most', SpinSecondsField),
-        ('is', SpinSecondsField),
-    ]),
-    ('Date added', [
-        ('in the last', SpinDateField),
-        ('not in the last', SpinDateField),
-    ]),
-    ('Last played', [
-        ('in the last', SpinDateField),
-        ('not in the last', SpinDateField),
-    ]),
-    ('Location', [
-        ('is', EntryField),
-        ('is not', EntryField),
-        ('contains', EntryField),
-        ('does not contain', EntryField),
-        ('regex', EntryField),
-        ('not regex', EntryField),
-    ]),
-    (_('BPM'), [
-        ('is', EntryField),
-        ('less than', SpinNothing),
-        ('greater than', SpinNothing),
-        ('between', EntryAndEntryField),
-        ('is set', NullField),
-        ('is not set', NullField),
-    ]),
-    (_('Grouping'),[
-        ('is', EntryField),
-        ('is not', EntryField),
-        ('contains', EntryField),
-        ('does not contain', EntryField),
-        ('regex', EntryField),
-        ('not regex', EntryField),
-        ('is set', NullField),
-        ('is not set', NullField),
-    ]),
+   
+    ('Playlist', [
+        ('Track is in', PlaylistField),
+        ('Track not in', PlaylistField),
+    ])
 ]
 
 # NOTE: We use N_ (fake gettext) because these strings are translated later by
@@ -228,24 +213,43 @@ _TRANS = {
     N_('in the last'): '>=',
     # TRANSLATORS: Example: track has not been added in the last 5 hours
     N_('not in the last'): '<',
+    # TRANSLATORS: True if a track is contained in the specified playlist
+    N_('Track is in'): 'pin',
+    # TRANSLATORS: True if a track is not contained in the specified playlist
+    N_('Track not in'): '!pin',
 }
 
+# This table is a reverse lookup for the actual tag name from a display
+# name.
+# This gets populated below. Only add special tags/searches here.
 _NMAP = {
-    N_('Artist'): 'artist',
-    N_('Title'): 'title',
-    N_('Album'): 'album',
-    N_('Length'): '__length',
-    N_('Rating'): '__rating',
-    N_('Plays'): '__playcount',
-    N_('Year'): 'date',
-    N_('Genre'): 'genre',
-    N_('Date added'): '__date_added',
-    N_('Last played'): '__last_played',
-    N_('Location'): '__loc',
-    N_('BPM'): 'bpm',
-    N_('Grouping'): 'grouping',
+    N_('Rating'): '__rating', # special
+    N_('Playlist'): '__playlist', # not a real tag
 }
 
+# update the tables based on the globally stored tag list
+def __update_maps():
+    
+    from xl.metadata.tags import tag_data
+    
+    for tag, data in tag_data.iteritems():
+        
+        if data is None:
+            continue
+        
+        # don't catch this KeyError -- if it fails, fix it!
+        criteria = __criteria_types[data.type]
+        
+        if criteria is None:
+            continue
+            
+        CRITERIA.append((data.name, criteria))
+        
+        _NMAP[data.name] = tag
+
+__update_maps()
+
+
 class TrackWrapper(object):
     def __init__(self, track, playlist):
         self.track = track
@@ -260,7 +264,6 @@ class TrackWrapper(object):
 
         if not text: return self.track.get_loc_for_io()
         return text
-        
 
 
 class BasePlaylistPanelMixin(gobject.GObject):
@@ -290,11 +293,21 @@ class BasePlaylistPanelMixin(gobject.GObject):
         self.track_image = icons.MANAGER.pixbuf_from_icon_name(
             'audio-x-generic', gtk.ICON_SIZE_SMALL_TOOLBAR)
 
-    def remove_selected_playlist(self):
+    def remove_playlist(self, ignored=None):
         """
             Removes the selected playlist from the UI
             and from the underlying manager
         """
+        
+        dialog = gtk.MessageDialog(None,
+            gtk.DIALOG_MODAL, gtk.MESSAGE_QUESTION, gtk.BUTTONS_YES_NO,
+            _("Are you sure you want to permanently delete the selected playlist?"))
+        response = dialog.run()
+        dialog.destroy()
+        
+        if response != gtk.RESPONSE_YES:
+            return 
+        
         selected_playlist = self.tree.get_selected_page(raw=True)
         if selected_playlist is not None:
             if isinstance(selected_playlist, playlist.SmartPlaylist):
@@ -310,26 +323,39 @@ class BasePlaylistPanelMixin(gobject.GObject):
             (model, iter) = selection.get_selected()
             self.model.remove(iter)
 
-    def rename_selected_playlist(self, name):
+    def rename_playlist(self, playlist):
         """
-            Renames the selected playlist
-
-            @param name: the new name
+            Renames the playlist
         """
+        
+        if playlist is None:
+            return
+        
+        # Ask for new name
+        dialog = dialogs.TextEntryDialog(
+            _("Enter the new name you want for your playlist"),
+            _("Rename Playlist"), playlist.name)
+        
+        result = dialog.run()
+        name = dialog.get_value()
+        
+        dialog.destroy()
+        
+        if result != gtk.RESPONSE_OK or name == '':
+            return
+                
         if name in self.playlist_manager.playlists:
             # name is already in use
             dialogs.error(self.parent, _("The "
                 "playlist name you entered is already in use."))
             return
 
-        playlist = self.tree.get_selected_page()
-        if playlist is not None:
-            old_name = playlist.name
-            selection = self.tree.get_selection()
-            (model, iter) = selection.get_selected()
-            model.set_value(iter, 1, name)
-            #Update the manager aswell
-            self.playlist_manager.rename_playlist(playlist, name)
+        selection = self.tree.get_selection()
+        (model, iter) = selection.get_selected()
+        model.set_value(iter, 1, name)
+        
+        # Update the manager aswell
+        self.playlist_manager.rename_playlist(playlist, name)
 
     def open_selected_playlist(self):
         selection = self.tree.get_selection()
@@ -362,7 +388,11 @@ class BasePlaylistPanelMixin(gobject.GObject):
                 playlist.SmartPlaylist)):
                 # for smart playlists
                 if hasattr(item, 'get_playlist'):
-                    item = item.get_playlist(self.collection)
+                    try:
+                        item = item.get_playlist(self.collection)
+                    except Exception as e:
+                        dialogs.error(self.parent, _("Error loading smart playlist: %s") % str(e))
+                        return
                 else:
                     #Get an up to date copy
                     item = self.playlist_manager.get_playlist(item.name)
@@ -516,13 +546,13 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
     ui_info = ('playlists.ui', 'PlaylistsPanelWindow')
 
     def __init__(self, parent, playlist_manager,
-        smart_manager, collection):
+        smart_manager, collection, name):
         """
             Intializes the playlists panel
 
             @param playlist_manager:  The playlist manager
         """
-        panel.Panel.__init__(self, parent)
+        panel.Panel.__init__(self, parent, name)
         BasePlaylistPanelMixin.__init__(self)
         self.playlist_manager = playlist_manager
         self.smart_manager = smart_manager
@@ -573,11 +603,12 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
             'music-library', gtk.ICON_SIZE_SMALL_TOOLBAR)
 
         # menus
-        self.playlist_menu = menu.PlaylistsPanelPlaylistMenu()
-        self.smart_menu = menu.PlaylistsPanelPlaylistMenu(smart=True)
-        self.default_menu = menu.PlaylistsPanelMenu()
-        self.track_menu = menu.PlaylistsPanelTrackMenu()
-
+        self.playlist_menu = menus.PlaylistsPanelPlaylistMenu(self)
+        self.smart_menu = menus.PlaylistsPanelPlaylistMenu(self)
+        self.default_menu = menus.PlaylistPanelMenu(self)
+        
+        self.track_menu = menus.TrackPanelMenu(self)
+        
         self._connect_events()
         self._load_playlists()
 
@@ -587,45 +618,6 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
 
         self.tree.connect('key-release-event', self.on_key_released)
 
-        self.track_menu.connect('remove-track', lambda *e:
-            self.remove_selected_track())
-        self.smart_menu.connect('properties', lambda *e:
-            self._playlist_properties())
-
-        for item in ('playlist', 'smart', 'default'):
-            menu = getattr(self, '%s_menu' % item)
-            menu.connect('add-playlist', lambda *e:
-                self.add_new_playlist())
-            menu.connect('add-smart-playlist', lambda *e:
-                self.add_smart_playlist())
-            menu.connect('import-playlist', lambda *e:
-                self.import_playlist())
-
-            if item != 'default':
-                menu.connect('append-items', lambda *e:
-                    self.emit('append-items', self.tree.get_selected_tracks(), 
-                        False))
-                menu.connect('replace-items', lambda *e:
-                    self.emit('replace-items', self.tree.get_selected_tracks()))
-                menu.connect('queue-items', lambda *e:
-                    self.emit('queue-items', self.tree.get_selected_tracks()))
-                menu.connect('rating-changed', self.on_rating_changed)
-
-                menu.connect('open-playlist', lambda *e:
-                    self.open_selected_playlist())
-                menu.connect('export-playlist', lambda widget:
-                    self.export_selected_playlist())
-                menu.connect('export-playlist-files', lambda widget, path:
-                    self.export_selected_playlist_files(path))
-                menu.connect('rename-playlist', lambda widget, name:
-                    self.rename_selected_playlist(name))
-                menu.connect('remove-playlist', lambda *e:
-                    self.remove_selected_playlist())
-
-            if item == 'smart':
-                menu.connect('edit-playlist', lambda *e:
-                    self.edit_selected_smart_playlist())
-
     def _playlist_properties(self):
         pl = self.tree.get_selected_page(raw=True)
         if isinstance(pl, playlist.SmartPlaylist):
@@ -719,11 +711,12 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
             Shows a dialog to ask the user to import a new playlist
         """
         
-        def _on_playlist_selected(dialog, playlist):
-            self.add_new_playlist( playlist, playlist.name )
+        def _on_playlists_selected(dialog, playlists):
+            for playlist in playlists:
+                self.add_new_playlist( playlist, playlist.name )
         
         dialog = dialogs.PlaylistImportDialog()
-        dialog.connect('playlist-selected', _on_playlist_selected)
+        dialog.connect('playlists-selected', _on_playlists_selected)
         dialog.show()
                 
     def add_smart_playlist(self):
@@ -783,6 +776,15 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
         """
             Shows a dialog for editing the currently selected smart playlist
         """
+        pl = self.tree.get_selected_page(raw=True)
+        self.edit_smart_playlist(pl)
+        
+    def edit_smart_playlist(self, pl):
+        """
+            Shows a dialog for editing a smart playlist
+        """
+        if not isinstance(pl, playlist.SmartPlaylist): return
+
         _REV = {}
         for k, v in _TRANS.iteritems():
             _REV[v] = k
@@ -791,9 +793,6 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
         for k, v in _NMAP.iteritems():
             _REV_NMAP[v] = k
 
-        pl = self.tree.get_selected_page(raw=True)
-        if not isinstance(pl, playlist.SmartPlaylist): return
-
         params = pl.search_params
         state = []
 
@@ -847,8 +846,7 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
                     return False
                 except ValueError:
                     pass # playlist didn't exist
-
-            self.smart_manager.remove_playlist(pl.name)
+            
             pl = playlist.SmartPlaylist(name, self.collection)
             pl.set_or_match(matchany)
             pl.set_return_limit(limit)
@@ -859,6 +857,7 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
                 value = item[1]
                 pl.add_param(_NMAP[field], _TRANS[op], value)
 
+            self.smart_manager.remove_playlist(pl.name)
             self.smart_manager.save_playlist(pl)
 
             selection = self.tree.get_selection()
@@ -1100,31 +1099,44 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
                 self.tree.enable_model_drag_dest([self.playlist_target],
                                                      gtk.gdk.ACTION_DEFAULT)
 
-    def export_selected_playlist(self):
+    # 
+    #  TODO: these should be moved somewhere more general
+    # 
+
+    def export_playlist(self, pl):
         """
             Exports the selected playlist to path
 
             @path where we we want it to be saved, with a
                 valid extension we support
         """
-        playlist = self.tree.get_selected_page()
+        
         if playlist is not None:
-            dialog = dialogs.PlaylistExportDialog(playlist)
+            dialog = dialogs.PlaylistExportDialog(pl)
             dialog.show()
                     
-    def export_selected_playlist_files(self, uri):
+    def export_playlist_files(self, pl):
         '''
-            Exports the selected playlist files to URI
+            Exports the playlist files to a URI
             
             @uri where we want it to be saved
         '''
-        pl = self.tree.get_selected_page()
-        if pl is not None:
+        
+        if pl is None:
+            return 
+        
+        def _on_uri(uri):
             pl_files = [track.get_loc_for_io() for track in pl]
             dialog = dialogs.FileCopyDialog( pl_files, uri, 
                 _('Exporting %s') % pl.name )
             dialog.do_copy()
             
+        dialog = dialogs.DirectoryOpenDialog(title=_('Choose directory to export files to'))
+        dialog.set_select_multiple(False)
+        dialog.connect( 'uris-selected', lambda widget, uris: _on_uri(uris[0]))
+        dialog.run()
+        dialog.destroy()
+            
 
     def on_key_released(self, widget, event):
         """
@@ -1172,14 +1184,7 @@ class PlaylistsPanel(panel.Panel, BasePlaylistPanelMixin):
                 #menu we will show
                 if isinstance(pl, playlist.Playlist) or \
                     isinstance(pl, playlist.SmartPlaylist):
-                    dialog = gtk.MessageDialog(None,
-                        gtk.DIALOG_MODAL, gtk.MESSAGE_QUESTION, 
-                        gtk.BUTTONS_YES_NO,
-                        _("Are you sure you want to permanently delete "
-                        "the selected playlist?"))
-                    if dialog.run() == gtk.RESPONSE_YES:
-                        self.remove_selected_playlist()
-                    dialog.destroy()
+                    self.remove_playlist(pl)
                 elif isinstance(pl, TrackWrapper):
                     self.remove_selected_track()
             return True
@@ -1226,6 +1231,10 @@ class PlaylistDragTreeView(DragTreeView):
         DragTreeView.__init__(self, container, receive, source)
         self.show_cover_drag_icon = False
 
+    def get_selection_empty(self):
+        '''Returns True if there are no selected items'''
+        return self.get_selection().count_selected_rows() == 0
+
     def get_selected_tracks(self):
         """
             Used by the menu, just basically gets the selected
@@ -1276,7 +1285,10 @@ class PlaylistDragTreeView(DragTreeView):
         # for smart playlists
         if isinstance(item, playlist.SmartPlaylist):
             if raw: return item
-            return item.get_playlist(self.container.collection)
+            try:
+                return item.get_playlist(self.container.collection)
+            except:
+                return None
         if isinstance(item, radio.RadioItem):
             if raw: return item
             return item.get_playlist()
