@@ -35,6 +35,7 @@ import os.path
 from xl import (
     metadata, 
     providers,
+    settings,
     xdg
 )
 from xl.common import clamp
@@ -89,6 +90,13 @@ def info(parent, message=None, markup=None):
         dialog.set_markup(markup)
     dialog.run()
     dialog.destroy()
+    
+def yesno(parent, message):
+    '''Gets a Yes/No response from a user'''
+    dlg = gtk.MessageDialog(parent=parent, type=gtk.MESSAGE_QUESTION, buttons=gtk.BUTTONS_YES_NO, message_format=message)
+    response = dlg.run() 
+    dlg.destroy()
+    return response
 
 class AboutDialog(object):
     """
@@ -696,7 +704,7 @@ class DirectoryOpenDialog(gtk.FileChooserDialog):
     }
     _last_location = None
 
-    def __init__(self, parent=None, title=_('Choose Directory to Open')):
+    def __init__(self, parent=None, title=_('Choose Directory to Open'), select_multiple=True):
         gtk.FileChooserDialog.__init__(self,
             title,
             parent=parent,
@@ -707,7 +715,7 @@ class DirectoryOpenDialog(gtk.FileChooserDialog):
         self.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
         self.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
         self.set_local_only(False)
-        self.set_select_multiple(True)
+        self.set_select_multiple(select_multiple)
 
         self.connect('response', self.on_response)
 
@@ -752,7 +760,7 @@ class PlaylistImportDialog(gtk.FileChooserDialog):
         A dialog for importing a playlist
     """
     __gsignals__ = {
-        'playlist-selected': (
+        'playlists-selected': (
             gobject.SIGNAL_RUN_LAST,
             gobject.TYPE_BOOLEAN,
             (gobject.TYPE_PYOBJECT,),
@@ -775,7 +783,7 @@ class PlaylistImportDialog(gtk.FileChooserDialog):
 
         self.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
         self.set_local_only(False)
-        self.set_select_multiple(False)
+        self.set_select_multiple(True)
 
         playlist_filter = gtk.FileFilter()
         playlist_filter.set_name(_('Playlist Files'))
@@ -829,13 +837,21 @@ class PlaylistImportDialog(gtk.FileChooserDialog):
         if response == gtk.RESPONSE_OK:
             PlaylistImportDialog._last_location = self.get_current_folder_uri()
             
-            try:
-                playlist = import_playlist(self.get_uri())
-            except InvalidPlaylistTypeError, e:
-                error(None, 'Invalid playlist: %s' % e)
-                self.destroy()
-            else:
-                self.emit('playlist-selected', playlist)
+            playlists = []
+            for uri in self.get_uris():            
+                try:
+                    playlists.append(import_playlist(uri))
+                except InvalidPlaylistTypeError, e:
+                    error(None, 'Invalid playlist "%s": %s' % (uri, e))
+                    self.destroy()
+                    return
+                except Exception, e:
+                    error(None, 'Invalid playlist "%s": (internal error): %s' % (uri, e))
+                    self.destroy()
+                    return
+            
+            self.emit('playlists-selected', playlists)
+                    
 
         #self.destroy()
 
@@ -1530,3 +1546,44 @@ def ask_for_playlist_name(playlist_manager, name=None):
         else:
             return name
 
+def save(parent, output_fname, output_setting=None, extensions=None, title=_("Save As")):
+    """
+        A 'save' dialog utility function, which can be used to easily
+        remember the last location the user saved something.
+        
+        :param parent:          Parent window
+        :param output_fname:    Output filename
+        :param output_setting:  Setting to store the last 'output directory' saved at
+        :param extensions:      Valid output extensions. Dict { '.m3u': 'Description', .. }
+        :param title:           Title of dialog
+        
+        :returns: None if user cancels, chosen URI otherwise
+    """
+    
+    uri = None
+    
+    dialog = FileOperationDialog(title, parent,
+            gtk.FILE_CHOOSER_ACTION_SAVE,
+            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+            gtk.STOCK_SAVE, gtk.RESPONSE_ACCEPT))
+    
+    if extensions is not None:
+        dialog.add_extensions(extensions)
+    
+    dialog.set_current_name(output_fname)
+    
+    if output_setting:
+        output_dir = settings.get_option(output_setting)
+        if output_dir:
+            dialog.set_current_folder_uri(output_dir)
+        
+    if dialog.run() == gtk.RESPONSE_ACCEPT:
+        uri = dialog.get_uri()
+        
+        settings.set_option(output_setting, dialog.get_current_folder_uri())
+        
+    dialog.destroy()
+        
+    return uri
+    
+
