@@ -1,3 +1,5 @@
+# Filter Widget/Dialog
+# Copyright (C) 2006 Johannes Sasongko <sasongko@gmail.com>
 # Copyright (C) 2008-2010 Adam Olsen
 #
 # This program is free software; you can redistribute it and/or modify
@@ -12,7 +14,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #
 #
 # The developers of the Exaile media player hereby grant permission
@@ -32,6 +34,7 @@ and Rhythmbox's automatic playlists.
 
 import gobject
 import gtk
+import urllib
 
 from xl.nls import gettext as _
 
@@ -65,7 +68,7 @@ class FilterDialog(gtk.Dialog):
         self.vbox.pack_start(top, False)
         top.show_all()
 
-        self.filter = f = FilterWidget(criteria)
+        self.filter = f = FilterWidget(sorted(criteria, key=lambda k: _(k[0])))
         f.add_row()
         f.set_border_width(5)
         self.vbox.pack_start(f)
@@ -245,6 +248,9 @@ class FilterWidget(gtk.Table):
 
         criterion = Criterion(self.criteria)
         criterion.show()
+        
+        if len(self.rows) != 0:
+            criterion.set_state(self.rows[-1][0].get_state())
 
         remove_btn = gtk.Button()
         image = gtk.Image()
@@ -340,6 +346,8 @@ class Criterion(gtk.HBox):
             self.child = field_class()
         else:
             self.combo = combo = gtk.combo_box_new_text()
+            if len(subcriteria) > 10:
+                self.combo.set_wrap_width(5)
             self.subcriteria = subcriteria
             for subc in subcriteria:
                 combo.append_text(_(subc[0]))
@@ -395,6 +403,25 @@ class Criterion(gtk.HBox):
 
 # Sample fields
 
+class ComboEntryField(gtk.HBox):
+    '''Select from multiple fixed values, but allow the user to enter text'''
+    
+    def __init__(self, values):
+        gtk.HBox.__init__(self)
+        
+        self.combo = gtk.combo_box_entry_new_text()
+        for value in values:
+            self.combo.append_text(value)
+        
+        self.pack_start(self.combo)
+        self.combo.show()
+    
+    def get_state(self):
+        return self.combo.get_child().get_text()
+    
+    def set_state(self, state):
+        self.combo.get_child().set_text(state)
+
 class NullField(gtk.HBox):
     '''Used as a placeholder for __null__ values'''
     
@@ -447,6 +474,16 @@ class EntryField(gtk.Entry):
         if type(state) == list or type(state) == tuple:
             state = state[0]
         self.set_text(unicode(state))
+        
+class QuotedEntryField(gtk.Entry):
+    def __init__(self):
+        gtk.Entry.__init__(self)
+    def get_state(self):
+        return unicode(urllib.quote(self.get_text()), 'utf-8')
+    def set_state(self, state):
+        if type(state) == list or type(state) == tuple:
+            state = state[0]
+        self.set_text(unicode(urllib.unquote(state)))
 
 class EntryLabelEntryField(MultiEntryField):
     def __init__(self, label):
