@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #
 #
 # The developers of the Exaile media player hereby grant permission
@@ -363,6 +363,10 @@ class MarkerManager(providers.ProviderHandler):
         """
             Triggers "reached" signal of markers
         """
+        
+        if player.current is None:
+            return
+        
         track_length = player.current.get_tag_raw('__length')
 
         if track_length is None:
